<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:svg="http://www.w3.org/2000/svg"
                xmlns:xlink="http://www.w3.org/1999/xlink"
                xmlns:a3="http://ns.adobe.com/AdobeSVGViewerExtensions/3.0/"
                exclude-result-prefixes="svg">

<!--

XSLT transform generating SVG display from XML source
by Wendell Piez [wendell@piez.org]
   January 2002
   rev 2003-01-22
   12:23 AM
This stylesheet adapted specifically for Jim Surkamp's "Antietam Soldiers"
works also for Surkamp's "Constitutional Convention"

-->

<xsl:output method="xml" indent="yes"/>

<xsl:strip-space elements="*"/>

<xsl:variable name="margin" select="0"/>

<xsl:param name="vertical-offset" select="0"/>


<xsl:variable name="ecmascript">

<script a3:scriptImplementation="Adobe" type="text/ecmascript">
<![CDATA[

var newMenuRoot = parseXML(printNode(document.getElementById('ConventionMenu')), contextMenu);
contextMenu.replaceChild(newMenuRoot,contextMenu.firstChild);

function view(viewBox) {

var viewframe = svgDocument.getElementById('root');
viewframe.setAttribute('viewBox',viewBox);

}

]]>
</script>

</xsl:variable>

<xsl:variable name="css">
  <xsl:text>

  </xsl:text>
  <xsl:value-of select="/*/style[@type='text/css']"/>

</xsl:variable>

<xsl:variable name="stylesheetdefs">
  <xsl:call-template name="buildmenus"/>
  <clipPath id="cameoclip">
    <ellipse rx="48%" ry="48%" cx="50%" cy="50%"/>
  </clipPath>
</xsl:variable>

<xsl:variable name="SVGincluded" select="document(//svg/@source)"/>

<xsl:variable name="images" select="document(/*/imagebase/@src)//image"/>

<xsl:variable name="textbase" select="document(/*/textbase/@src)"/>

<xsl:variable name="views" select="/*/views/choice"/>

<xsl:template match="*|@*">
  <!-- by default we assume all elements and attributes not otherwise matched
       are SVG and should be copied through -->
  <xsl:copy>
    <xsl:apply-templates select="node()|@*"/>
  </xsl:copy>
</xsl:template>

<!-- templates -->

<xsl:template match="/">
  <xsl:comment>
    <xsl:text>&#xA;  SVG generated from XML source</xsl:text>
    <xsl:text>&#xA; XSLT transform by Wendell Piez </xsl:text>
  </xsl:comment>
  <xsl:apply-templates/>
</xsl:template>

<xsl:template match="screen">
  <xsl:param name="linkbox">
    <xsl:call-template name="linkbox"/>
  </xsl:param>
  <xsl:param name="content">
    <xsl:apply-templates/>
  </xsl:param>
  <xsl:param name="width" select="@width"/>
  <xsl:param name="height" select="@height"/>
  <xsl:variable name="full-width" select="$width"/>
  <xsl:variable name="full-height" select="$height"/>
  <xsl:variable name="view-width" select="$full-width"/>
  <xsl:variable name="view-height" select="$full-height"/>

  <svg width="{$view-width}" height="{$view-height}">
    <xsl:choose>
      <xsl:when test="ancestor::screen">
        <xsl:attribute name="x">0</xsl:attribute>
        <xsl:attribute name="y">0</xsl:attribute>
        <xsl:attribute name="viewBox">
          <xsl:text>0 0 </xsl:text>
          <xsl:value-of select="$full-width"/>
          <xsl:text> </xsl:text>
          <xsl:value-of select="$full-height"/>
        </xsl:attribute>
      </xsl:when>
      <xsl:otherwise>

        <!-- here goes the buttons for particular views
        <set attributeName="viewBox" attributeType="XML"
             to="300 400 320 240" begin="testzoom.click" fill="freeze"/> -->
      <xsl:attribute name="id">root</xsl:attribute>
      <xsl:attribute name="scriptImplementation" namespace="http://ns.adobe.com/AdobeSVGViewerExtensions/3.0/">Adobe</xsl:attribute>

        <defs>
          <xsl:copy-of select="$ecmascript"/>
          <style type="text/css">
            <xsl:value-of select="$css"/>
            <xsl:copy-of select="$SVGincluded//svg:defs/svg:style/node()"/>
            <xsl:copy-of select="//defs/style/node()"/>
          </style>
          <xsl:copy-of select="//defs/*[not(self::style)]"/>
          <xsl:copy-of select="$SVGincluded//svg:defs/*[not(self::style)]"/>
          <xsl:copy-of select="$stylesheetdefs"/>
        </defs>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:copy-of select="$content"/>
    <xsl:copy-of select="$linkbox"/>
  </svg>
</xsl:template>

<xsl:template name="screen">
  <xsl:param name="content">
    <xsl:apply-templates/>
  </xsl:param>
  <xsl:param name="width" select="@width"/>
  <xsl:param name="height" select="@height"/>

  <svg width="{$width}" height="{$height}"
       viewBox="0 0 {$width} {$height}"
       x="0" y="0">
    <xsl:copy-of select="$content"/>
  </svg>
</xsl:template>

<xsl:template match="style|defs|imagebase|textbase|views"/>

<xsl:template match="cameo">
  <xsl:variable name="src" select="translate(shot/@source,'#','')"/>
  <xsl:if test="not($images[@id=$src])">
    <xsl:message>Warning: image "<xsl:value-of select="$src"/>" not found...</xsl:message>
  </xsl:if>
  <g transform="translate({@xOffset - 4} {@yOffset - 6})"
     id="{@who}-cameo" style="visibility: visible">
        <set attributeName="visibility" attributeType="CSS"
             to="hidden" begin="mapview.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="visible" begin="namesview.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="hidden" begin="{@who}-cameo.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="visible" begin="{@who}-tablet.click" fill="freeze"/>
    <xsl:call-template name="portrait">
        <xsl:with-param name="width" select="54"/>
        <xsl:with-param name="height" select="72"/>
    </xsl:call-template>
    <xsl:variable name="captionoffset">
      <xsl:choose>
        <xsl:when test="@cast='left'">0</xsl:when>
        <xsl:otherwise>
          <xsl:value-of select="54"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:variable>
    <g transform="translate({$captionoffset} 36)">
      <xsl:apply-templates select="label" mode="caption"/>
    </g>
  </g>
  <g transform="translate({@xOffset + 5} {@yOffset + 7.5})"
     id="{@who}-medal" style="visibility: hidden">
        <set attributeName="visibility" attributeType="CSS"
             to="visible" begin="mapview.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="hidden" begin="namesview.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="hidden" begin="{@who}-medal.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="visible" begin="{@who}-cameo.click" fill="freeze"/>
      <xsl:call-template name="portrait">
        <xsl:with-param name="width" select="30"/>
        <xsl:with-param name="height" select="40"/>
        <xsl:with-param name="flag" select="'medal'"/>
      </xsl:call-template>
      <!-- xsl:apply-templates select="label" mode="label-only"/ -->
  </g>
  <g transform="translate({@xOffset} {@yOffset})"
     id="{@who}-tablet" style="visibility: hidden">
        <set attributeName="visibility" attributeType="CSS"
             to="hidden" begin="mapview.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="hidden" begin="namesview.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="hidden" begin="{@who}-tablet.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="visible" begin="{@who}-medal.click" fill="freeze"/>
    <xsl:call-template name="tablet-box"/>
    <xsl:call-template name="portrait">
      <xsl:with-param name="width" select="45"/>
      <xsl:with-param name="height" select="60"/>
      <xsl:with-param name="flag" select="'tablet'"/>
    </xsl:call-template>
    <xsl:call-template name="tablet-text"/>
  </g>
</xsl:template>

<xsl:template name="portrait">
  <xsl:param name="width" select="45"/>
  <xsl:param name="height" select="60"/>
  <xsl:param name="flag" select="'cameo'"/>
  <xsl:call-template name="screen">
    <xsl:with-param name="width" select="$width"/>
    <xsl:with-param name="height" select="$height"/>
    <xsl:with-param name="content">
      <g style="clip-path:url(#cameoclip)">
        <xsl:apply-templates select="shot">
          <xsl:with-param name="screenwidth" select="$width"/>
          <xsl:with-param name="screenheight" select="$height"/>
        </xsl:apply-templates>
      </g>
      <ellipse class="{$flag}-portraitframe-{@class}" rx="48%" ry="48%" cx="50%" cy="50%"/>
    </xsl:with-param>
  </xsl:call-template>
</xsl:template>

<xsl:template name="medalportrait">
  <xsl:variable name="width" select="30"/>
  <xsl:variable name="height" select="40"/>
  <xsl:call-template name="portrait">
    <xsl:with-param name="width" select="$width"/>
    <xsl:with-param name="height" select="$height"/>
    <xsl:with-param name="flag" select="'medal'"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="tabletportrait">
  <xsl:variable name="width" select="45"/>
  <xsl:variable name="height" select="60"/>
  <xsl:call-template name="portrait">
    <xsl:with-param name="width" select="$width"/>
    <xsl:with-param name="height" select="$height"/>
    <xsl:with-param name="flag" select="'tablet'"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="label" mode="label-only">
  <text class="label">
    <xsl:apply-templates/>
  </text>
</xsl:template>

<xsl:template match="label" mode="caption">
  <xsl:variable name="text-anchor">
    <xsl:choose>
      <xsl:when test="../@cast='left'">end</xsl:when>
      <xsl:otherwise>start</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <text class="caption-{../@class}" text-anchor="{$text-anchor}">
    <xsl:apply-templates/>
  </text>
</xsl:template>

<xsl:template name="tablet-box">
  <xsl:param name="height" select="80"/>
  <!-- context node is a cameo -->
  <xsl:variable name="boxwidth" select="(($height * 4) div 3)"/>
  <xsl:variable name="xOffset">
    <xsl:choose>
      <xsl:when test="@cast='left'">
        <xsl:value-of select="-5 - ($boxwidth * 0.5)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="-5"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <rect x="{$xOffset}" y="-15" width="{$boxwidth}" height="{$height}"
        class="tablet-bgbox-{@class}"/>
</xsl:template>

<xsl:template name="tablet-text">
  <xsl:param name="height" select="80"/>
  <!-- context node is a cameo -->
  <xsl:variable name="boxwidth" select="(($height * 4) div 3)"/>
  <xsl:variable name="text-anchor">
    <xsl:choose>
      <xsl:when test="@cast='left'">start</xsl:when>
      <xsl:otherwise>end</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="textalignX">
    <xsl:choose>
      <xsl:when test="@cast='left'">
        <xsl:value-of select="0 - ($boxwidth  * 0.5)"/>
      </xsl:when>
      <xsl:otherwise>
        <xsl:value-of select="$boxwidth - 10"/>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <g class="tablet-text-{@class}" text-anchor="{$text-anchor}" transform="translate({$textalignX} -5)">
    <xsl:apply-templates select="tablet"/>
  </g>
</xsl:template>

<xsl:template match="tablet">
  <xsl:apply-templates select="name"/>
  <xsl:variable name="lineheight" select="8"/>
  <g transform="translate(0 {1.5 * $lineheight})">
    <xsl:apply-templates select="state"/>
  </g>
  <g transform="translate(0 {3 * $lineheight})">
      <xsl:apply-templates select="line">
        <xsl:with-param name="lineheight" select="$lineheight"/>
      </xsl:apply-templates>
  </g>
</xsl:template>

<xsl:template match="name">
  <text class="name"><xsl:apply-templates/></text>
</xsl:template>

<xsl:template match="state">
  <text class="state"><xsl:value-of select="."/></text>
</xsl:template>

<xsl:template match="line">
  <xsl:param name="lineheight" select="8"/>
  <text x="0" y="{$lineheight * count(preceding-sibling::line)}" class="line">
    <xsl:apply-templates/>
  </text>
</xsl:template>

<xsl:template match="shot[@zoom]">
  <xsl:variable name="src" select="translate(@source,'#','')"/>
  <xsl:variable name="image" select="$images[@id=$src]"/>
  <xsl:variable name="width" select="$image/@width"/>
  <xsl:variable name="height" select="$image/@height"/>
  <xsl:variable name="screenwidth" select="ancestor::screen/@width"/>
  <xsl:variable name="screenheight" select="ancestor::screen/@height"/>
  <!-- X and Y offsets are fancy to get things to center -->
  <xsl:variable name="xOffset">
    <xsl:choose>
      <xsl:when test="@xCast"><xsl:value-of select="0 - @xCast"/></xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="yOffset">
    <xsl:choose>
      <xsl:when test="@yCast"><xsl:value-of select="0 - @yCast"/></xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:variable name="xPos" select="$xOffset + ($screenwidth div 2) - (($width * @zoom) div 2)"/>
  <xsl:variable name="yPos" select="$yOffset + ($screenheight div 2) - (($height * @zoom) div 2)"/>
  <xsl:call-template name="paintimage">
    <xsl:with-param name="src" select="$src"/>
    <xsl:with-param name="xPos" select="$xPos"/>
    <xsl:with-param name="yPos" select="$yPos"/>
    <xsl:with-param name="height" select="$height * @zoom"/>
    <xsl:with-param name="width" select="$width * @zoom"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="paintimage">
  <xsl:param name="src" select="''"/>
  <xsl:param name="xPos" select="0"/>
  <xsl:param name="yPos" select="0"/>
  <xsl:param name="height" select="0"/>
  <xsl:param name="width" select="0"/>
  <xsl:variable name="image" select="$images[@id=$src]"/>
  <xsl:if test="$image">
    <image x="{$xPos}" y="{$yPos}" height="{$height}" width="{$width}" xlink:href="{$image/@src}"/>
    <xsl:if test="@subdue">
      <rect x="0" y="0" width="100%" height="100%" fill="white" fill-opacity="{@subdue}">
        <set attributeName="visibility" attributeType="CSS"
             to="hidden" begin="mapview.click" fill="freeze"/>
        <set attributeName="visibility" attributeType="CSS"
             to="visible" begin="namesview.click" fill="freeze"/>
      </rect>
    </xsl:if>
  </xsl:if>
</xsl:template>

<xsl:template match="shot[@zoom='fit'] | cameo/shot" priority="20" name="fitshot">
  <xsl:param name="screenwidth" select="(ancestor::screen|ancestor::cameo)[last()]/@width"/>
  <xsl:param name="screenheight" select="(ancestor::screen|ancestor::cameo)[last()]/@height"/>
  <xsl:variable name="src" select="translate(@source,'#','')"/>
  <xsl:variable name="image" select="$images[@id=$src]"/>
  <xsl:variable name="ratio" select="$image/@width div $image/@height"/>
  <xsl:variable name="screenratio" select="$screenwidth div $screenheight"/>
  <xsl:choose>
    <xsl:when test="$screenratio > $ratio">
      <xsl:call-template name="fitXshot">
        <xsl:with-param name="screenwidth" select="$screenwidth"/>
        <xsl:with-param name="screenheight" select="$screenheight"/>
      </xsl:call-template>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="fitYshot">
        <xsl:with-param name="screenwidth" select="$screenwidth"/>
        <xsl:with-param name="screenheight" select="$screenheight"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<xsl:template match="shot[@zoom='fitX']" priority="10" name="fitXshot">
  <xsl:param name="screenwidth" select="(ancestor::screen|ancestor::cameo)[last()]/@width"/>
  <xsl:param name="screenheight" select="(ancestor::screen|ancestor::cameo)[last()]/@height"/>
  <xsl:variable name="src" select="translate(@source,'#','')"/>
  <xsl:variable name="image" select="$images[@id=$src]"/>
  <xsl:variable name="ratio" select="$image/@width div $image/@height"/>
  <xsl:variable name="shownheight" select="$screenwidth div $ratio"/>
  <xsl:variable name="shownY" select="($screenheight - $shownheight) div 2"/>
  <xsl:variable name="xOffset">
    <xsl:choose>
      <xsl:when test="@xCast"><xsl:value-of select="0 - @xCast"/></xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="yOffset">
    <xsl:choose>
      <xsl:when test="@yCast"><xsl:value-of select="0 - @yCast"/></xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:call-template name="paintimage">
    <xsl:with-param name="src" select="$src"/>
    <xsl:with-param name="xPos" select="$xOffset"/>
    <xsl:with-param name="yPos" select="$shownY + $yOffset"/>
    <xsl:with-param name="height" select="$shownheight"/>
    <xsl:with-param name="width" select="$screenwidth"/>
  </xsl:call-template>
</xsl:template>

<xsl:template match="shot[@zoom='fitY']" priority="10" name="fitYshot">
  <xsl:param name="screenwidth" select="(ancestor::screen|ancestor::cameo)[last()]/@width"/>
  <xsl:param name="screenheight" select="(ancestor::screen|ancestor::cameo)[last()]/@height"/>
  <xsl:variable name="src" select="translate(@source,'#','')"/>
  <xsl:variable name="image" select="$images[@id=$src]"/>
  <xsl:variable name="ratio" select="$image/@width div $image/@height"/>
  <xsl:variable name="shownwidth" select="$screenheight * $ratio"/>
  <xsl:variable name="shownX" select="($screenwidth - $shownwidth) div 2"/>
  <xsl:variable name="xOffset">
    <xsl:choose>
      <xsl:when test="@xCast"><xsl:value-of select="0 - @xCast"/></xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>
  <xsl:variable name="yOffset">
    <xsl:choose>
      <xsl:when test="@yCast"><xsl:value-of select="0 - @yCast"/></xsl:when>
      <xsl:otherwise>0</xsl:otherwise>
    </xsl:choose>
  </xsl:variable>

  <xsl:call-template name="paintimage">
    <xsl:with-param name="src" select="$src"/>
    <xsl:with-param name="xPos" select="$shownX + $xOffset"/>
    <xsl:with-param name="yPos" select="$yOffset"/>
    <xsl:with-param name="height" select="$screenheight"/>
    <xsl:with-param name="width" select="$shownwidth"/>
  </xsl:call-template>
</xsl:template>

<xsl:template name="linkbox">
  <xsl:if test="not(ancestor::screen)">
    <!-- here's zoom buttons
    <rect id="testzoom" x="20" y="558" width="50" height="50" class="mapview"/> -->
    <rect id="mapview" x="20" y="628" width="50" height="50" class="mapview"/>
    <rect id="namesview" x="20" y="698" width="50" height="50" class="namesview"/>
  </xsl:if>
</xsl:template>

<xsl:template name="buildmenus">
<menu id="ConventionMenu" xmlns="http://piez.org/spectacula" onload="GetPosition(evt)">

  <header>Constitutional Convention</header>

  <xsl:apply-templates select="$views" mode="menu"/>
  <separator/>
  <item action="ZoomIn" id="ZoomIn">Zoom In</item>
  <item action="ZoomOut" id="ZoomOut">Zoom Out</item>
  <item action="OriginalView" id="OriginalView">Original View</item>
  <item onactivate="view('0 0 1024 768')" id="fullview">Full screen</item>

  <separator/>
  <menu>
  <header>Adobe SVG</header>
  <item action="Quality" id="Quality">Higher Quality</item>
  <item action="Pause" id="Pause">Pause</item>
  <item action="Mute" id="Mute">Mute</item>
  <separator/>
  <item action="Find" id="Find">Find...</item>
  <item action="FindAgain" id="FindAgain">Find Again</item>
  <separator/>

  <item action="Copy" id="Copy">Copy Selected Text</item>
  <item action="CopySVG" id="CopySVG">Copy SVG</item>
  <item action="ViewSVG" id="ViewSVG">View SVG</item>
  <item action="ViewSource" id="ViewSource">View Source</item>
  <item action="SaveAs" id="SaveAs">Save SVG As...</item>
  </menu>

  <separator/>

  <item action="Help" id="Help">Help</item>
  <item action="About" id="About">About Adobe SVG Viewer...</item>

</menu>

</xsl:template>

<xsl:template match="choice" mode="menu">
  <xsl:variable name="x" select="@xOffset"/>
  <xsl:variable name="y" select="@yOffset"/>
  <xsl:variable name="width" select="@width"/>
  <xsl:variable name="height" select="$width * 0.75"/>
  <item onactivate="view('{$x} {$y} {$width} {$height}')" id="{@id}">
    <xsl:value-of select="."/>
  </item>
</xsl:template>

</xsl:stylesheet>